@@ -69,6 +69,44 @@ open_mmap_file(char *path, uint32_t file_flags)
 	return open(path, file_flags, 0600);
 }
 
+#if defined(QB_BSD) || !defined(HAVE_POSIX_FALLOCATE)
+static int local_fallocate(int fd, size_t bytes)
+{
+	long page_size = sysconf(_SC_PAGESIZE);
+	long write_size = QB_MIN(page_size, bytes);
+	char *buffer = NULL;
+	int i;
+	size_t written;
+
+	if (page_size < 0) {
+		goto error_exit;
+	}
+	buffer = calloc(1, write_size);
+	if (buffer == NULL) {
+		goto error_exit;
+	}
+
+	for (i = 0; i < (bytes / write_size); i++) {
+	retry_write:
+		written = write(fd, buffer, write_size);
+		if (written == -1 && errno == EINTR) {
+			goto retry_write;
+		}
+		if (written != write_size) {
+			free(buffer);
+			errno = ENOSPC;
+			goto error_exit;
+		}
+	}
+	free(buffer);
+
+	return 0;
+
+error_exit:
+	return -1;
+}
+#endif
+
 int32_t
 qb_sys_mmap_file_open(char *path, const char *file, size_t bytes,
 		       uint32_t file_flags)
@@ -81,6 +119,9 @@ qb_sys_mmap_file_open(char *path, const char *file, size_t bytes,
 	int32_t i;
 #endif
 	char *is_absolute = strchr(file, '/');
+#ifdef HAVE_POSIX_FALLOCATE
+	int32_t fallocate_retry = 5;
+#endif
 
 	if (is_absolute) {
 		(void)strlcpy(path, file, PATH_MAX);
@@ -121,38 +162,37 @@ qb_sys_mmap_file_open(char *path, const char *file, size_t bytes,
 	}
 #endif
 #ifdef HAVE_POSIX_FALLOCATE
-	if ((res = posix_fallocate(fd, 0, bytes)) != 0) {
-		errno = res;
-		res = -1 * res;
-		qb_util_perror(LOG_ERR, "couldn't allocate file %s", path);
-		goto unlink_exit;
-	}
+	/* posix_fallocate(3) can be interrupted by a signal,
+	   so retry few times before giving up */
+	do {
+		fallocate_retry--;
+		res = posix_fallocate(fd, 0, bytes);
+		if (res == EINTR) {
+			qb_util_log(LOG_DEBUG, "got EINTR trying to allocate file %s, retrying...", path);
+			continue;
+#ifdef QB_BSD
+		} else if (res == EINVAL) { /* posix_fallocate() fails on ZFS
+					       https://lists.freebsd.org/pipermail/freebsd-current/2018-February/068448.html */
+			qb_util_log(LOG_DEBUG, "posix_fallocate returned EINVAL - running on ZFS?");
+			if (file_flags & O_CREAT) {
+				if (local_fallocate(fd, bytes)) {
+					goto unlink_exit;
+				}
+			}
+#endif
+		} else if (res != 0) {
+			errno = res;
+			res = -1 * res;
+			qb_util_perror(LOG_ERR, "couldn't allocate file %s", path);
+			goto unlink_exit;
+		}
+		break;
+	} while (fallocate_retry > 0);
 #else
 	if (file_flags & O_CREAT) {
-		long page_size = sysconf(_SC_PAGESIZE);
-		long write_size = QB_MIN(page_size, bytes);
-		if (page_size < 0) {
-			res = -errno;
+		if (local_fallocate(fd, bytes)) {
 			goto unlink_exit;
 		}
-		buffer = calloc(1, write_size);
-		if (buffer == NULL) {
-			res = -ENOMEM;
-			goto unlink_exit;
-		}
-		for (i = 0; i < (bytes / write_size); i++) {
-retry_write:
-			written = write(fd, buffer, write_size);
-			if (written == -1 && errno == EINTR) {
-				goto retry_write;
-			}
-			if (written != write_size) {
-				res = -ENOSPC;
-				free(buffer);
-				goto unlink_exit;
-			}
-		}
-		free(buffer);
 	}
 #endif /* HAVE_POSIX_FALLOCATE */
 
